using System;
using System.Collections.Generic;
using GTA;
using System.Text;

namespace PoliceScript
{
    class PoliceVehicle
    {
        public Car vehicle;
        public bool dead;
        public float initialHealth;
        public Blip marker;
    }

    public class CWPoliceScript : Script
    {
        List<PoliceVehicle> vehicles;
        List<PoliceVehicle> deaths;
        List<PoliceVehicle> blinkingCars;
        int vehicleWantedLevel = 0;
        int vehicleCounter = 0;

        public CWPoliceScript()
        {
            this.OnTick += Tick;
            BindConsoleCommand("cwp_wantedlevel", new ConsoleCommandDelegate(SetWanted), this);

            vehicles = new List<PoliceVehicle>();
            deaths = new List<PoliceVehicle>();
            blinkingCars = new List<PoliceVehicle>();

            Interval = 500;
        }

        public void SetWanted(string[] args)
        {
            int wanted = PlayerObject.WantedLevel;

            try
            {
                wanted = int.Parse(args[args.Length - 1]);
            }
            catch
            {
            }


            PlayerObject.WantedLevel = wanted;
        }

        public void Tick(object sender, EventArgs e)
        {
            if (PlayerObject.WantedLevel == 0)
            {
                if (vehicles.Count > 0 || blinkingCars.Count > 0)
                {
                    foreach (PoliceVehicle vehicle in vehicles)
                    {
                        vehicle.marker.Delete();
                        vehicle.vehicle.NoLongerNeeded();
                    }

                    foreach (PoliceVehicle vehicle in blinkingCars)
                    {
                        vehicle.marker.Delete();
                        vehicle.vehicle.NoLongerNeeded();
                    }

                    vehicles.Clear();
                    blinkingCars.Clear();
                    deaths.Clear();
                }

                return;
            }

            BuildPoliceVehicleArray(PlayerObject.Character.Position);

            if (PlayerObject.WantedLevel != vehicleWantedLevel)
            {
                vehicleWantedLevel = PlayerObject.WantedLevel;
                vehicleCounter = vehicleWantedLevel;

                Game.DisplayTextBox(string.Format("Wanted level. {0} vehicles to go.", vehicleCounter));
            }

            CheckVehicleExistence(PlayerObject.Character.Position);

            foreach (PoliceVehicle vehicle in blinkingCars)
            {
                if (!vehicle.vehicle.IsAlive)
                {
                    continue;
                }

                Vector3 vehposition = vehicle.vehicle.Position;
                Vector3 Difference = new Vector3(30, 30, 0);
                Vector3 Position = PlayerObject.Character.Position;

                if ((vehposition.pX <= (Position - Difference).pX || vehposition.pY <= (Position - Difference).pY) ||
                    (vehposition.pX >= (Position + Difference).pX || vehposition.pY >= (Position + Difference).pY))
                {
                    continue;
                }


                //Initialize.Log("Blinking vehicle with id " + vehicle.vehicle.Handle.ToString());

                BlipColor newColor;

                if (vehicle.marker.Color == BlipColor.Int_0)
                {
                    newColor = BlipColor.Int_1;
                }
                else
                {
                    newColor = BlipColor.Int_0;
                }

                vehicle.marker.Delete();

                Blip blip = Blip.AddBlip(vehicle.vehicle);
                blip.Size = 1;
                blip.Color = newColor;
                vehicle.marker = blip;
            }

            if (vehicleCounter <= 0)
            {
                PlayerObject.WantedLevel = vehicleWantedLevel - 1;

                if (PlayerObject.WantedLevel == 0)
                {
                    foreach (PoliceVehicle vehicle in vehicles)
                    {
                        vehicle.marker.Delete();
                        vehicle.vehicle.NoLongerNeeded();
                    }

                    foreach (PoliceVehicle vehicle in blinkingCars)
                    {
                        vehicle.marker.Delete();
                        vehicle.vehicle.NoLongerNeeded();
                    }

                    vehicles.Clear();
                    blinkingCars.Clear();
                    deaths.Clear();
                }
            }
        }

        private void CheckVehicleExistence(Vector3 Position)
        {
            Vector3 Difference = new Vector3(200, 200, 0);

            List<PoliceVehicle> removes = new List<PoliceVehicle>();
            //List<PoliceVehicle> deaths = new List<PoliceVehicle>();

            foreach (PoliceVehicle vehicle in vehicles)
            {
                if (!vehicle.vehicle.IsAlive)
                {
                    removes.Add(vehicle);
                    continue;
                }
                
                Vector3 vehposition = vehicle.vehicle.Position;

                if ((vehposition.pX <= (Position - Difference).pX || vehposition.pY <= (Position - Difference).pY) ||
                    (vehposition.pX >= (Position + Difference).pX || vehposition.pY >= (Position + Difference).pY))
                {
                    removes.Add(vehicle);
                }

                if (!deaths.Contains(vehicle))
                {
                    if (vehicle.vehicle.Health <= vehicle.initialHealth)
                    {
                        //Initialize.Log("So that really DOES work. Current difference: " + (vehicle.initialHealth - vehicle.vehicle.Health).ToString());
                    }

                    bool specialKill = false;

                    if (vehicle.vehicle.Model == CarID.HPV1000)
                    {
                        GTA.Char driver = vehicle.vehicle.Driver;

                        if (!driver.IsAlive && (vehicle.initialHealth - vehicle.vehicle.Health) >= 10)
                        {
                            specialKill = true;
                        }
                    }

                    if ((vehicle.vehicle.Health + 55) <= vehicle.initialHealth || specialKill)
                    {
                        deaths.Add(vehicle);
                        //vehicle.dead = true;
                        vehicleCounter -= 1;
                        Game.DisplayTextBox(string.Format("Another vehicle down! {0} to go.", vehicleCounter));
                    }
                }
            }

            foreach (PoliceVehicle vehicle in removes)
            {
                if (vehicle.vehicle.IsAlive)
                {
                    vehicle.vehicle.NoLongerNeeded();
                }
                vehicle.marker.Delete();
                vehicles.Remove(vehicle);
            }

            foreach (PoliceVehicle vehicle in deaths)
            {
                if (blinkingCars.Contains(vehicle))
                {
                    vehicle.marker.Delete();
                    blinkingCars.Remove(vehicle);
                }
            }
        }

        private void BuildPoliceVehicleArray(Vector3 Position)
        {
            Vector3 Difference = new Vector3(80, 80, 0);

            for (int i = 0; i <= 6; i++)
            {
                CarID carID = CarID.PoliceLosSantos;

                switch (i)
                {
                    case 0:
                        carID = CarID.PoliceLosSantos;
                        break;
                    case 1:
                        carID = CarID.PoliceLasVenturas;
                        break;
                    case 2:
                        carID = CarID.PoliceSanFierro;
                        break;
                    case 3:
                        carID = CarID.PoliceRanger;
                        break;
                    case 4:
                        carID = CarID.HPV1000;
                        break;
                    case 5:
                        carID = CarID.FBIRancher;
                        break;
                    case 6:
                        carID = CarID.Barracks;
                        break;
                }

                Car vehicle = Car.GetRandomCar(Position + Difference, Position - Difference, carID);

                //Initialize.Log(string.Format("Coordinates are near: {0} {1} {2} {3}", (Position + Difference).pX, (Position + Difference).pY, (Position - Difference).pX, (Position - Difference).pY));
                //Initialize.Log("Checking vehicle. Vehicle ID: " + vehicle.Handle.ToString());

                if (/*vehicle.Handle != 0xFFFFFFFF && */vehicle.IsAlive)
                {
                    bool toContinue = false;

                    foreach (PoliceVehicle veh in vehicles)
                    {
                        if (veh.vehicle == vehicle)
                        {
                            toContinue = true;
                        }
                    }

                    if (toContinue)
                    {
                        continue;
                    }

                    Blip blip = Blip.AddBlip(vehicle);
                    blip.Color = BlipColor.Int_0;
                    blip.Size = 1;

                    PoliceVehicle vdata = new PoliceVehicle();
                    vdata.vehicle = vehicle;
                    vdata.initialHealth = vehicle.Health;
                    vdata.dead = false;
                    vdata.marker = blip;

                    vehicles.Add(vdata);
                    blinkingCars.Add(vdata);

                    //Initialize.Log("Police vehicle found. Vehicle ID: " + vehicle.Handle.ToString());

                    //Game.DisplayTextBox("Found police vehicle.");
                }
            }
            //Car.GetRandomCar(
        }
    }
}